/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * queue.h - struture definations for queue objects
 *
 * Include Files Requried:
 *
 *	<sys/types.h>
 *	"attribute.h"
 *	"list_link.h"
 *      "server_limits.h"
 *      "resource.h"
 *
 * Queue Types
 */

#ifndef QUEUE_H
#define QUEUE_H

#define QTYPE_Unset	0
#define QTYPE_Execution 1
#define QTYPE_RoutePush 2
#define QTYPE_RoutePull 3

/*
 * Attributes, including the various resource-lists are maintained in an
 * array in a "decoded or parsed" form for quick access to the value.
 *
 * The following enum defines the index into the array.
 */

enum queueattr {
	QA_ATR_QType,
	QA_ATR_Priority,
	QA_ATR_MaxJobs,
	QA_ATR_TotalJobs,
	QA_ATR_JobsByState,
	QA_ATR_MaxRun,
	QA_ATR_AclHostEnabled,
	QA_ATR_AclHost,
	QA_ATR_AclUserEnabled,
	QA_ATR_AclUsers,
	QA_ATR_FromRouteOnly,
	QA_ATR_ResourceMax,
	QA_ATR_ResourceMin,
	QA_ATR_ResourceDefault,

	/* The following attributes apply only to exection queues */

	QE_ATR_AclGroupEnabled,
	QE_ATR_AclGroup,
	QE_ATR_ChkptMim,
	QE_ATR_RendezvousRetry,
	QE_ATR_ReservedExpedite,
	QE_ATR_ReservedSync,
	QE_ATR_ResourceAvail,
	QE_ATR_ResourceAssn,
	QE_ATR_KillDelay,
	QE_ATR_MaxUserRun,
	QE_ATR_MaxGrpRun,

	/* The following attribute apply only to routing queues... */

	QR_ATR_RouteDestin,
	QR_ATR_AltRouter,
	QR_ATR_RouteHeld,
	QR_ATR_RouteWaiting,
	QR_ATR_RouteRetryTime,
	QR_ATR_RouteLifeTime,

#include "site_que_attr_enum.h"
	QA_ATR_Enabled,	/* these are last for qmgr print function   */
	QA_ATR_Started,
	QA_ATR_LAST	/* WARNING: Must be the highest valued enum */
};

extern attribute_def que_attr_def[];

	
/* at last we come to the queue definition itself	*/
 
struct pbs_queue {
	list_link	qu_link;		/* forward/backward links */
	list_head	qu_jobs;		/* jobs in this queue */

	struct queuefix {
		int	qu_modified;		/* != 0 => update disk file */
		int	qu_type;		/* queue type: exec, route */
		time_t	qu_ctime;		/* time queue created */
		time_t	qu_mtime;		/* time queue last modified */
		char	qu_name[PBS_MAXQUEUENAME]; /* queue name */
	} qu_qs;

	int	qu_numjobs;			/* current numb jobs in queue */
	int	qu_njstate[PBS_NUMJOBSTATE];	/* # of jobs per state */
	char	qu_jobstbuf[100];

	/* the queue attributes */

	attribute	qu_attr[QA_ATR_LAST];
};
typedef struct pbs_queue pbs_queue;

extern pbs_queue *find_queuebyname A_((char *));
extern pbs_queue *get_dfltque();
extern pbs_queue *que_alloc A_((char *name));
extern void   que_free A_((pbs_queue *));
extern pbs_queue *que_recov A_((char *));
extern int    que_save A_((pbs_queue *));

#endif	/* QUEUE_H */
